@@ -33,18 +33,17 @@ def run_weld(
             }
 
     with run_beet(config, directory=directory) as ctx:
-        ctx.require(partial(load_packs, packs=packs, pack_types=pack_types))
+        ctx.require(partial(load_packs, packs=list(packs), pack_types=pack_types))
 
         yield ctx
 
 
 def load_packs(
     ctx: Context,
-    packs: Iterable[str] | Iterable[ZipFile],
+    packs: list[str] | list[ZipFile],
     pack_types: list[Literal["data_pack", "resource_pack"]],
 ):
     # ctx.require("weld.setup")
-    ctx.meta["weld"] = {"packs": packs}
     for pack in packs:
         for pack_type in pack_types:
             match pack:
@@ -63,21 +62,27 @@ def load_packs(
                     else:
                         ctx.assets.load(file)
 
-        if len(pack_types) > 1:
-            ctx.require(add_fabric_mod_json)
+    if len(pack_types) > 1:
+        JsonFile(FABRIC_MOD_TEMPLATE.render(
+            pack_hash=hash(tuple(packs)),
+            pack_names=[
+                pack.filename if type(pack) is ZipFile else pack for pack in packs
+            ],
+            mc_version=ctx.minecraft_version,
+        ))
+        ctx.require(partial(add_fabric_mod_json, packs=packs))
 
 
-def add_fabric_mod_json(ctx: Context):
-    with ctx.override():
-        ctx.data.extra["fabric.mod.json"] = JsonFile(
-            FABRIC_MOD_TEMPLATE.render(
-                packs=(packs := ctx.meta["weld"]["packs"]),
-                pack_names=[
-                    pack if type(pack) is str else pack.filename for pack in packs
-                ],
-                mc_version=ctx.minecraft_version,
-            )
+def add_fabric_mod_json(ctx: Context, packs: list[str] | list[ZipFile]):
+    ctx.data.extra["fabric.mod.json"] = JsonFile(
+        FABRIC_MOD_TEMPLATE.render(
+            packs=packs,
+            pack_names=[
+                pack.filename if type(pack) is ZipFile else pack for pack in packs
+            ],
+            mc_version=ctx.minecraft_version,
         )
+    )
 
 
 def print_pack_name(ctx: Context):
